"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.readmeBaseurl = exports.missingSections = exports.semverWorkflowPreset = void 0;
const inject_1 = require("../actions/inject");
const resolveLatestVersions_1 = require("../npm/resolveLatestVersions");
const package_1 = require("../npm/package");
const log_1 = __importDefault(require("../util/log"));
const outdent_1 = __importDefault(require("outdent"));
const chalk_1 = __importDefault(require("chalk"));
const path_1 = __importDefault(require("path"));
const files_1 = require("../util/files");
const errorToUndefined_1 = require("../util/errorToUndefined");
exports.semverWorkflowPreset = {
    name: 'semver-workflow',
    description: 'Files and dependencies for conventional-commits, github workflow and semantic-release.',
    apply: applyPreset,
};
const info = (write, msg, ...args) => write && log_1.default.info(msg, ...args);
function applyPreset(options) {
    return __awaiter(this, void 0, void 0, function* () {
        yield (0, inject_1.writeAssets)(semverWorkflowFiles(), options);
        yield addPrepareScript(options);
        yield addDevDependencies(options);
        yield updateReadme(options);
    });
}
function addPrepareScript(options) {
    return __awaiter(this, void 0, void 0, function* () {
        const pkg = yield (0, package_1.getPackage)(options);
        const didWrite = yield (0, package_1.addPackageJsonScripts)(pkg, options, (scripts) => {
            scripts.prepare = (0, package_1.addScript)(`husky install`, scripts.prepare);
            return scripts;
        });
        info(didWrite, 'Added prepare script to package.json');
    });
}
function addDevDependencies(options) {
    return __awaiter(this, void 0, void 0, function* () {
        const pkg = yield (0, package_1.getPackage)(options);
        const devDeps = (0, package_1.sortKeys)(Object.assign(Object.assign({}, pkg.devDependencies), (yield semverWorkflowDependencies())));
        const newPkg = Object.assign({}, pkg);
        newPkg.devDependencies = devDeps;
        yield (0, package_1.writePackageJsonDirect)(newPkg, options);
        log_1.default.info('Updated devDependencies.');
        log_1.default.info(chalk_1.default.green((0, outdent_1.default) `
        semantic-release preset injected.

        Please confer
        https://github.com/sanity-io/plugin-kit/blob/main/docs/presets.md#semver-workflow
        to finalize configuration for this preset.
  `.trim()));
    });
}
function updateReadme(options) {
    var _a;
    return __awaiter(this, void 0, void 0, function* () {
        const { basePath } = options;
        const readmePath = path_1.default.join(basePath, 'README.md');
        const readme = (_a = (yield (0, files_1.readFile)(readmePath, 'utf8').catch(errorToUndefined_1.errorToUndefined))) !== null && _a !== void 0 ? _a : '';
        const { v3Banner, install, usage, developFooter } = yield readmeSnippets(options);
        const prependSections = missingSections(readme, [v3Banner, install, usage]);
        const appendSections = missingSections(readme, [developFooter]);
        if (prependSections.length || appendSections.length) {
            const updatedReadme = [...prependSections, readme, ...appendSections]
                .filter(Boolean)
                .join('\n\n');
            yield (0, files_1.writeFile)(readmePath, updatedReadme, { encoding: 'utf8' });
            log_1.default.info('Updated README. Please review the changes.');
        }
    });
}
function readmeSnippets(options) {
    return __awaiter(this, void 0, void 0, function* () {
        const pkg = yield (0, package_1.getPackage)(options);
        const bestEffortUrl = readmeBaseurl(pkg);
        const v3Banner = (0, outdent_1.default) `
    > **NOTE**
    >
    > This is the **Sanity Studio v3 version** of ${pkg.name}.
    >
    > For the v2 version, please refer to the [v2-branch](${bestEffortUrl}).
  `;
        const install = (0, outdent_1.default) `
    ## Installation

    \`\`\`
    npm install --save ${pkg.name}@studio-v3
    \`\`\`

    or

    \`\`\`
    yarn add ${pkg.name}@studio-v3
    \`\`\`
  `;
        const usage = (0, outdent_1.default) `
    ## Usage
  `;
        const developFooter = (0, outdent_1.default) `
    ## License

    MIT-licensed. See LICENSE.

    ## Develop & test

    This plugin uses [@sanity/plugin-kit](https://github.com/sanity-io/plugin-kit)
    with default configuration for build & watch scripts.

    See [Testing a plugin in Sanity Studio](https://github.com/sanity-io/plugin-kit#testing-a-plugin-in-sanity-studio)
    on how to run this plugin with hotreload in the studio.

    ### Release new version

    Run ["CI & Release" workflow](${bestEffortUrl}/actions/workflows/main.yml).
    Make sure to select the main branch and check "Release new version".

    Semantic release will only release on configured branches, so it is safe to run release on any branch.
  `;
        return {
            v3Banner,
            install,
            usage,
            developFooter,
        };
    });
}
/**
 * Returns sections that does not exists "close enough" in readme
 */
function missingSections(readme, sections) {
    return sections.filter((section) => !closeEnough(section, readme));
}
exports.missingSections = missingSections;
/**
 * a and b are considered "close enough" if > 50% of a lines exist in b lines
 * @param a
 * @param b
 */
function closeEnough(a, b) {
    const aLines = a.split('\n');
    const bLines = b.split('\n');
    const matchingLines = aLines.filter((line) => bLines.find((bLine) => bLine === line)).length;
    const isCloseEnough = matchingLines > aLines.length * 0.5;
    return isCloseEnough;
}
function semverWorkflowFiles() {
    return [
        { from: ['.github', 'workflows', 'main.yml'], to: ['.github', 'workflows', 'main.yml'] },
        { from: ['.husky', 'commit-msg'], to: ['.husky', 'commit-msg'] },
        { from: ['.husky', 'pre-commit'], to: ['.husky', 'pre-commit'] },
        { from: ['.releaserc.json'], to: '.releaserc.json' },
        { from: ['npmrc'], to: '.npmrc' },
        { from: ['commitlint.template.js'], to: 'commitlint.config.js' },
        { from: ['lint-staged.template.js'], to: 'lint-staged.config.js' },
    ].map((fromTo) => (Object.assign(Object.assign({}, fromTo), { from: ['semver-workflow', ...fromTo.from] })));
}
function semverWorkflowDependencies() {
    return __awaiter(this, void 0, void 0, function* () {
        return (0, resolveLatestVersions_1.resolveLatestVersions)([
            '@commitlint/cli',
            '@commitlint/config-conventional',
            '@sanity/semantic-release-preset',
            'husky',
            'lint-staged',
        ]);
    });
}
function readmeBaseurl(pkg) {
    var _a, _b, _c;
    return ((_c = (_b = (_a = pkg.repository) === null || _a === void 0 ? void 0 : _a.url) !== null && _b !== void 0 ? _b : pkg.homepage) !== null && _c !== void 0 ? _c : 'TODO')
        .replaceAll(/.+:\/\//g, 'https://')
        .replaceAll(/\.git/g, '')
        .replaceAll(/git@github.com\//g, 'github.com/')
        .replaceAll(/git@github.com:/g, 'https://github.com/')
        .replaceAll(/#.+/g, '');
}
exports.readmeBaseurl = readmeBaseurl;
//# sourceMappingURL=semver-workflow.js.map