"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.findStudioV3Config = exports.hasSanityJson = exports.getReferencesPartPaths = exports.validatePaths = exports.validatePluginManifest = exports.validateManifest = exports.readManifest = exports.getPaths = void 0;
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const util_1 = __importDefault(require("util"));
const package_json_1 = __importDefault(require("../../package.json"));
const buildExtensions_1 = require("../configs/buildExtensions");
const files_1 = require("../util/files");
const errorToUndefined_1 = require("../util/errorToUndefined");
const stat = util_1.default.promisify(fs_1.default.stat);
const readFile = util_1.default.promisify(fs_1.default.readFile);
const allowedPartProps = ['name', 'implements', 'path', 'description'];
const disallowedPluginProps = ['api', 'project', 'plugins', 'env'];
function getPaths(options) {
    return __awaiter(this, void 0, void 0, function* () {
        const { basePath } = options;
        const manifest = yield readManifest(options);
        if (!manifest.paths) {
            return null;
        }
        return absolutifyPaths(manifest.paths, basePath);
    });
}
exports.getPaths = getPaths;
function absolutifyPaths(paths, basePath) {
    const getPath = (relative) => relative ? path_1.default.resolve(path_1.default.join(basePath, relative)) : undefined;
    return paths
        ? {
            basePath,
            compiled: getPath(paths.compiled),
            source: getPath(paths.source),
        }
        : { basePath };
}
function readManifest(options) {
    return __awaiter(this, void 0, void 0, function* () {
        const { basePath, validate = true } = options;
        const manifestPath = path_1.default.normalize(path_1.default.join(basePath, 'sanity.json'));
        let content;
        try {
            content = yield readFile(manifestPath, 'utf8');
        }
        catch (err) {
            if (err.code === 'ENOENT') {
                throw new Error(`No sanity.json found. sanity.json is required for plugins to function. Use \`${package_json_1.default.binname} init\` for a new plugin, or create an empty \`sanity.json\` with an empty object (\`{}\`) for existing ones.`);
            }
            throw new Error(`Failed to read "${manifestPath}": ${err.message}`);
        }
        let parsed;
        try {
            parsed = JSON.parse(content);
        }
        catch (err) {
            throw new Error(`Error parsing "${manifestPath}": ${err.message}`);
        }
        if (validate) {
            yield validateManifest(parsed, options);
        }
        return parsed;
    });
}
exports.readManifest = readManifest;
function validateManifest(manifest, opts) {
    return __awaiter(this, void 0, void 0, function* () {
        const options = Object.assign({ isPlugin: true }, opts);
        if (!isObject(manifest)) {
            throw new Error(`Invalid sanity.json: Root must be an object`);
        }
        if (options.isPlugin) {
            yield validatePluginManifest(manifest, options);
        }
        else {
            yield validateProjectManifest(manifest);
        }
        if ('root' in manifest && typeof manifest.root !== 'boolean') {
            throw new Error(`Invalid sanity.json: "root" property must be a boolean if declared`);
        }
        yield validateParts(manifest, Object.assign(Object.assign({}, options), { paths: absolutifyPaths(manifest.paths, options.basePath) }));
    });
}
exports.validateManifest = validateManifest;
function validateProjectManifest(manifest) {
    if ('paths' in manifest) {
        throw new Error(`Invalid sanity.json: "paths" property has no meaning in a project manifest`);
    }
}
function validatePluginManifest(manifest, options) {
    return __awaiter(this, void 0, void 0, function* () {
        const disallowed = Object.keys(manifest)
            .filter((key) => disallowedPluginProps.includes(key))
            .map((key) => `"${key}"`);
        if (disallowed.length > 0) {
            const plural = disallowed.length > 1 ? 's' : '';
            const joined = disallowed.join(', ');
            throw new Error(`Invalid sanity.json: Key${plural} ${joined} ${plural ? 'are' : 'is'} not allowed in a plugin manifest`);
        }
        if (manifest.root) {
            throw new Error(`Invalid sanity.json: "root" cannot be truthy in a plugin manifest`);
        }
        yield validatePaths(manifest, options);
    });
}
exports.validatePluginManifest = validatePluginManifest;
function validatePaths(manifest, options) {
    return __awaiter(this, void 0, void 0, function* () {
        if (!('paths' in manifest)) {
            return;
        }
        if (!isObject(manifest.paths)) {
            throw new Error(`Invalid sanity.json: "paths" must be an object if declared`);
        }
        if (typeof manifest.paths.compiled !== 'string') {
            throw new Error(`Invalid sanity.json: "paths" must have a (string) "compiled" property if declared`);
        }
        if (typeof manifest.paths.source !== 'string') {
            throw new Error(`Invalid sanity.json: "paths" must have a (string) "source" property if declared`);
        }
        const sourcePath = path_1.default.resolve(options.basePath, manifest.paths.source);
        let srcStats;
        try {
            srcStats = yield stat(sourcePath);
        }
        catch (err) {
            if (err.code === 'ENOENT') {
                throw new Error(`sanity.json references "source" path which does not exist: "${sourcePath}"`);
            }
        }
        if (!(srcStats === null || srcStats === void 0 ? void 0 : srcStats.isDirectory())) {
            throw new Error(`sanity.json references "source" path which is not a directory: "${sourcePath}"`);
        }
    });
}
exports.validatePaths = validatePaths;
function validateParts(manifest, options) {
    return __awaiter(this, void 0, void 0, function* () {
        if (!('parts' in manifest)) {
            return;
        }
        if (!Array.isArray(manifest.parts)) {
            throw new Error(`Invalid sanity.json: "parts" must be an array if declared`);
        }
        let i = 0;
        for (const part of manifest.parts) {
            yield validatePart(part, i, options);
            i++;
        }
    });
}
function validatePart(part, index, options) {
    return __awaiter(this, void 0, void 0, function* () {
        if (!isObject(part)) {
            throw new Error(`Invalid sanity.json: "parts[${index}]" must be an object`);
        }
        validateAllowedPartKeys(part, index);
        validatePartStringValues(part, index);
        validatePartNames(part, index, options);
        yield validatePartFiles(part, index, options);
    });
}
function validatePartFiles(part, index, options) {
    return __awaiter(this, void 0, void 0, function* () {
        const { verifyCompiledParts, verifySourceParts, paths } = options;
        if (!(part === null || part === void 0 ? void 0 : part.path)) {
            return;
        }
        const ext = path_1.default.extname(part.path);
        if ((paths === null || paths === void 0 ? void 0 : paths.source) && ext && ext !== '.js' && buildExtensions_1.buildExtensions.includes(ext)) {
            throw new Error(`Invalid sanity.json: Part path has extension which is not applicable after compiling. ${ext} becomes .js after compiling. Specify filename without extension (${path_1.default.basename(part.path)}) (parts[${index}])`);
        }
        if (!verifySourceParts && !verifyCompiledParts) {
            return;
        }
        const [srcExists, libExists] = yield Promise.all([
            (0, files_1.hasSourceFile)(part.path, paths),
            verifyCompiledParts && (0, files_1.hasCompiledFile)(part.path, paths),
        ]);
        if (!srcExists) {
            throw new Error(`Invalid sanity.json: Part path references file that does not exist in source directory (${(paths === null || paths === void 0 ? void 0 : paths.source) || (paths === null || paths === void 0 ? void 0 : paths.basePath)}) (parts[${index}])`);
        }
        if (verifyCompiledParts && !libExists) {
            throw new Error(`Invalid sanity.json: Part path references file ("${part.path}") that does not exist in compiled directory (${paths === null || paths === void 0 ? void 0 : paths.compiled}) (parts[${index}])`);
        }
    });
}
function validatePartNames(part, index, options) {
    var _a, _b;
    const pluginName = options.pluginName ? options.pluginName.replace(/^sanity-plugin-/, '') : '';
    if (!(part === null || part === void 0 ? void 0 : part.name) || !((_a = part === null || part === void 0 ? void 0 : part.name) === null || _a === void 0 ? void 0 : _a.startsWith(`part:${pluginName}/`))) {
        throw new Error(`Invalid sanity.json: "name" must be prefixed with "part:${pluginName}/" - got "${part === null || part === void 0 ? void 0 : part.name}" (parts[${index}])`);
    }
    if (!((_b = part === null || part === void 0 ? void 0 : part.implements) === null || _b === void 0 ? void 0 : _b.startsWith('part:'))) {
        throw new Error(`Invalid sanity.json: "implements" must be prefixed with "part:" - got "${part === null || part === void 0 ? void 0 : part.implements}" (parts[${index}])`);
    }
}
function validateAllowedPartKeys(part, index) {
    const disallowed = Object.keys(part)
        .filter((key) => !allowedPartProps.includes(key))
        .map((key) => `"${key}"`);
    if (disallowed.length > 0) {
        const plural = disallowed.length > 1 ? 's' : '';
        const joined = disallowed.join(', ');
        throw new Error(`Invalid sanity.json: Key${plural} ${joined} ${plural ? 'are' : 'is'} not allowed in a part declaration (parts[${index}])`);
    }
}
function validatePartStringValues(part, index) {
    const nonStrings = Object.keys(part)
        .filter((key) => typeof part[key] !== 'string')
        .map((key) => `"${key}"`);
    if (nonStrings.length > 0) {
        const plural = nonStrings.length > 1 ? 's' : '';
        const joined = nonStrings.join(', ');
        throw new Error(`Invalid sanity.json: Key${plural} ${joined} should be of type string (parts[${index}])`);
    }
}
function isObject(obj) {
    return !Array.isArray(obj) && obj !== null && typeof obj === 'object';
}
function getReferencesPartPaths(manifest, basePath) {
    const { paths, parts = [] } = manifest;
    const compiledPath = path_1.default.resolve(basePath, (paths === null || paths === void 0 ? void 0 : paths.compiled) || '');
    return parts
        .filter((part) => part.path)
        .map((part) => part.path)
        .map((partPath) => (path_1.default.extname(partPath) === '' ? `${partPath}.js` : partPath))
        .map((partPath) => path_1.default.isAbsolute(partPath)
        ? partPath // Not sure if this ever happens, but :shrugs:
        : path_1.default.resolve(compiledPath, partPath));
}
exports.getReferencesPartPaths = getReferencesPartPaths;
function hasSanityJson(basePath) {
    return __awaiter(this, void 0, void 0, function* () {
        const file = yield (0, files_1.readJsonFile)(path_1.default.join(basePath, 'sanity.json')).catch(errorToUndefined_1.errorToUndefined);
        return { exists: Boolean(file), isRoot: Boolean(file && file.root) };
    });
}
exports.hasSanityJson = hasSanityJson;
function findStudioV3Config(basePath) {
    return __awaiter(this, void 0, void 0, function* () {
        const jsFile = 'sanity.config.js';
        const jsExists = yield (0, files_1.fileExists)(path_1.default.join(basePath, jsFile));
        if (jsExists) {
            return { v3ConfigFile: jsFile };
        }
        const tsFile = 'sanity.config.ts';
        const tsExists = yield (0, files_1.fileExists)(path_1.default.join(basePath, tsFile));
        return { v3ConfigFile: tsExists ? tsFile : undefined };
    });
}
exports.findStudioV3Config = findStudioV3Config;
//# sourceMappingURL=manifest.js.map